@@ -24,6 +24,7 @@
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "file-list.h"
 #include "utf8-fnmatch.h"
 
@@ -33,6 +34,7 @@
 
 /* -- filter -- */
  
+ 
 typedef enum {
         GNOME_VFS_DIRECTORY_FILTER_DEFAULT = 0,
         GNOME_VFS_DIRECTORY_FILTER_NODOTFILES = 1 << 1,
@@ -41,6 +43,7 @@ typedef enum {
 } GnomeVFSDirectoryFilterOptions;
 
 typedef struct {
+	char                            *pattern;
 	char                           **patterns;
 	int                              fnmatch_flags;
 	GnomeVFSDirectoryFilterOptions   options;
@@ -51,17 +54,18 @@ static Filter *
 filter_new (const gchar                    *pattern, 
 	    GnomeVFSDirectoryFilterOptions  options)
 {
-	Filter *new;
+	Filter *filter;
 
-        new = g_new (Filter, 1);
+        filter = g_new (Filter, 1);
 
-        new->patterns = search_util_get_patterns (pattern);
-        new->fnmatch_flags = 0;
-	new->options = options;
+	filter->pattern = g_strdup (pattern);
+        filter->patterns = search_util_get_patterns (pattern);
+        filter->fnmatch_flags = 0;
+	filter->options = options;
         if ((options & GNOME_VFS_DIRECTORY_FILTER_IGNORECASE) == GNOME_VFS_DIRECTORY_FILTER_IGNORECASE)
-                new->fnmatch_flags |= FNM_CASEFOLD;
+                filter->fnmatch_flags |= FNM_CASEFOLD;
 
-        return new;
+        return filter;
 }
 
 
@@ -80,6 +84,7 @@ filter_destroy (Filter *filter)
 {
         g_return_if_fail (filter != NULL);
 
+	g_free (filter->pattern);
 	if (filter->patterns != NULL)
 		g_strfreev (filter->patterns);
         g_free (filter);
@@ -641,10 +646,12 @@ path_list_async_interrupt (PathListHandle   *handle,
 
 
 struct _VisitDirData {
+	char             *base_dir;
 	char             *directory;
 	gboolean          recursive;
 	gboolean          follow_links;
 	gboolean          same_fs;
+	gboolean          include_directories;
 	VisitDirDoneFunc  done_func;
 	gpointer          done_data;
 	DoneFunc          interrupt_func;
@@ -657,8 +664,10 @@ struct _VisitDirData {
 	/* private */
 	
 	Filter           *filter;
+	GHashTable       *dirnames;
 };
 
+
 struct _VisitDirHandle {
 	VisitDirData   *vdd_data;
 };
@@ -670,6 +679,7 @@ visit_dir_data_free (VisitDirData *vdd)
 	if (vdd == NULL)
 		return;
 
+	g_free (vdd->base_dir);
 	g_free (vdd->directory);
 
 	if (vdd->dirs != NULL)
@@ -680,6 +690,9 @@ visit_dir_data_free (VisitDirData *vdd)
 	if (vdd->filter)
 		filter_destroy (vdd->filter);
 
+	if (vdd->dirnames)
+		g_hash_table_destroy (vdd->dirnames);
+
 	g_free (vdd);
 }
 
@@ -713,6 +726,48 @@ same_fs (const char *path1,
 }
 
 
+static GList*
+get_dir_list_from_file_list (VisitDirData *vdd, 
+			     GList        *files,
+			     gboolean      is_dir_list)
+{
+	GList *scan;
+	GList *dir_list = NULL;
+	
+	for (scan = files; scan; scan = scan->next) {
+		char *filename = scan->data;
+		char *dir_name;
+		
+		if (is_dir_list)
+			dir_name = g_strdup (filename + strlen (vdd->base_dir) + 1);
+		else
+			dir_name = remove_level_from_path (filename + strlen (vdd->base_dir) + 1);
+		
+		while ((dir_name != NULL) && (dir_name[0] != '\0') && (strcmp (dir_name, "/") != 0)) {
+			char *tmp;
+			char *dir;
+			
+			/* avoid to insert duplicated folders */
+
+			dir = g_strconcat (vdd->base_dir, "/", dir_name, NULL);
+			if (g_hash_table_lookup (vdd->dirnames, dir) == NULL) { 
+				g_hash_table_insert (vdd->dirnames, dir, GINT_TO_POINTER (1));
+				dir_list = g_list_prepend (dir_list, dir);
+			} else
+				g_free (dir);
+			
+			tmp = dir_name;
+			dir_name = remove_level_from_path (tmp);
+			g_free (tmp);
+		}
+		
+		g_free (dir_name);	
+	}
+	
+	return dir_list;
+}
+
+
 static void
 vd_path_list_done_cb (PathListData *pld, 
 		      gpointer      data)
@@ -759,15 +814,21 @@ vd_path_list_done_cb (PathListData *pld,
 				scan = scan->next;
 		}
 
+	if (vdd->include_directories)
+		vdd->files = g_list_concat (get_dir_list_from_file_list (vdd, pld->files, FALSE), vdd->files);
 	vdd->files = g_list_concat (pld->files, vdd->files);
 	pld->files = NULL;
+	
+	if (vdd->include_directories)
+		if (strcmp (vdd->filter->pattern, "*") == 0) 
+			vdd->files = g_list_concat (get_dir_list_from_file_list (vdd, pld->dirs, TRUE), vdd->files);
 
 	if (! vdd->recursive) {
 		if (vdd->done_func)
 			(* vdd->done_func) (vdd->files, vdd->done_data);
 		path_list_data_free (pld);
 		visit_dir_data_free (vdd);
-
+		
 		return;
 	}
 
@@ -779,7 +840,7 @@ vd_path_list_done_cb (PathListData *pld,
 		if (vdd->done_func)
 			(* vdd->done_func) (vdd->files, vdd->done_data);
 		visit_dir_data_free (vdd);
-
+		
 		return;
 	}
 
@@ -820,7 +881,8 @@ _visit_dir_async (const char   *dir,
 
 
 VisitDirHandle *
-visit_dir_async (const char       *directory, 
+visit_dir_async (VisitDirHandle   *handle,
+		 const char       *directory, 
 		 const char       *filter_pattern, 
 		 gboolean          recursive,
 		 gboolean          follow_links,
@@ -828,20 +890,23 @@ visit_dir_async (const char       *directory,
 		 gboolean          no_backup_files,
 		 gboolean          no_dot_files,
 		 gboolean          ignorecase,
+		 gboolean          include_directories,
 		 VisitDirDoneFunc  done_func,
 		 gpointer          done_data)
 {
 	VisitDirData                   *vdd;
 	GnomeVFSDirectoryFilterOptions  filter_options;
-	VisitDirHandle                 *handle;
 
 	vdd = g_new0 (VisitDirData, 1);
+	vdd->base_dir = g_strdup (directory);
 	vdd->directory = g_strdup (directory);
 	vdd->recursive = recursive;
+	vdd->include_directories = include_directories;
 	vdd->follow_links = follow_links;
 	vdd->same_fs = same_fs;
 	vdd->done_func = done_func;
 	vdd->done_data = done_data;
+	vdd->dirnames = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* file filter */
 
@@ -858,7 +923,8 @@ visit_dir_async (const char       *directory,
 
 	/* handle */
 
-	handle = g_new0 (VisitDirHandle, 1);
+	if (handle == NULL) 
+		handle = g_new0 (VisitDirHandle, 1);
 	handle->vdd_data = vdd;
 
 	_visit_dir_async (directory, vdd);
@@ -875,9 +941,9 @@ visit_dir_handle_free (VisitDirHandle   *handle)
 
 
 void
-visit_dir_async_interrupt    (VisitDirHandle   *handle,
-			      DoneFunc          f,
-			      gpointer          data)
+visit_dir_async_interrupt (VisitDirHandle   *handle,
+			   DoneFunc          f,
+			   gpointer          data)
 {
 	handle->vdd_data->interrupt_func = f;
 	handle->vdd_data->interrupt_data = data;
@@ -891,10 +957,18 @@ visit_dir_async_interrupt    (VisitDirHandle   *handle,
 
 
 typedef struct {
+	GList             *dir_list;
+	GList             *current_dir;
+	GList             *files;
+	VisitDirHandle    *handle;
+
 	char              *directory;
 	char              *base_dir;
+	gboolean           include_directories;
 	VisitDirDoneFunc   done_func;
 	gpointer           done_data;
+
+	guint              visit_timeout;
 } GetFileListData;
 
 
@@ -905,12 +979,16 @@ get_file_list_data_free (GetFileListData *gfl_data)
 	if (gfl_data == NULL)
 		return;
 
+	path_list_free (gfl_data->dir_list);
 	g_free (gfl_data->directory);
 	g_free (gfl_data->base_dir);
 	g_free (gfl_data);
 }
 
 
+/* -- get_wildcard_file_list_async -- */
+
+
 static void 
 get_wildcard_file_list_cb (GList *files, gpointer data)
 {
@@ -950,6 +1028,7 @@ get_wildcard_file_list_async  (const char       *directory,
 			       gboolean          no_backup_files,
 			       gboolean          no_dot_files,
 			       gboolean          ignorecase,
+			       gboolean          include_directories,
 			       VisitDirDoneFunc  done_func,
 			       gpointer          done_data)
 {
@@ -961,7 +1040,8 @@ get_wildcard_file_list_async  (const char       *directory,
 	gfl_data->done_func = done_func;
 	gfl_data->done_data = done_data;
 	
-	return visit_dir_async (directory,
+	return visit_dir_async (NULL,
+				directory,
 				filter_pattern, 
 				recursive,
 				follow_links,
@@ -969,11 +1049,15 @@ get_wildcard_file_list_async  (const char       *directory,
 				no_backup_files,
 				no_dot_files,
 				ignorecase,
+				include_directories,
 				get_wildcard_file_list_cb,
 				gfl_data);
 }
 
 
+/* -- get_directory_file_list_async -- */
+
+
 static void 
 get_directory_file_list_cb (GList *files, gpointer data)
 {
@@ -1011,6 +1095,7 @@ get_directory_file_list_cb (GList *files, gpointer data)
 VisitDirHandle *
 get_directory_file_list_async (const char       *directory,
 			       const char       *base_dir,
+			       gboolean          include_directories,
 			       VisitDirDoneFunc  done_func,
 			       gpointer          done_data)
 {
@@ -1022,6 +1107,7 @@ get_directory_file_list_async (const char       *directory,
 
 	gfl_data->directory = g_strdup (directory);
 	gfl_data->base_dir  = g_strdup (base_dir);
+	gfl_data->include_directories = include_directories;
 	gfl_data->done_func = done_func;
 	gfl_data->done_data = done_data;
 
@@ -1030,7 +1116,8 @@ get_directory_file_list_async (const char       *directory,
 	else
 		path = g_strconcat (base_dir, "/", directory, NULL);
 
-	handle = visit_dir_async (path,
+	handle = visit_dir_async (NULL,
+				  path,
 				  "*",
 				  TRUE,
 				  TRUE,
@@ -1038,9 +1125,138 @@ get_directory_file_list_async (const char       *directory,
 				  FALSE,
 				  FALSE,
 				  FALSE,
+				  include_directories,
 				  get_directory_file_list_cb,
 				  gfl_data);
 	g_free (path);
 
 	return handle;
 }
+
+
+/* -- get_items_file_list_async -- */
+
+
+static VisitDirHandle *visit_current_dir (GetFileListData *gfl_data);
+
+
+static gboolean
+visit_current_dir_idle_cb (gpointer data)
+{
+	GetFileListData *gfl_data = data;
+
+	g_source_remove (gfl_data->visit_timeout);
+	gfl_data->visit_timeout = 0;
+
+	visit_current_dir (gfl_data);
+
+	return FALSE;
+}
+
+
+static void 
+get_items_file_list_cb (GList *files, gpointer data)
+{
+	GetFileListData *gfl_data = data;
+
+	if (gfl_data->base_dir != NULL) {
+		GList *scan;
+		int    base_len;
+		
+		base_len = 0;
+		if (strcmp (gfl_data->base_dir, "/") != 0)
+			base_len = strlen (gfl_data->base_dir);
+
+		for (scan = files; scan; scan = scan->next) {
+			char *full_path = scan->data;
+
+			if (strncmp (gfl_data->base_dir, full_path, base_len) == 0) {
+				char *rel_path = g_strdup (full_path + base_len + 1);
+				gfl_data->files = g_list_prepend (gfl_data->files, rel_path);
+			}
+		}
+	}
+
+	gfl_data->current_dir = g_list_next (gfl_data->current_dir);
+	gfl_data->visit_timeout = g_idle_add (visit_current_dir_idle_cb, gfl_data);
+}
+
+
+static VisitDirHandle *
+visit_current_dir (GetFileListData *gfl_data)
+{
+	const char *directory;
+	char       *path;
+
+	if (gfl_data->current_dir == NULL) {
+		if (gfl_data->done_func) 
+			/* gfl_data->files must be deallocated in gfl_data->done_func */
+			gfl_data->done_func (gfl_data->files, gfl_data->done_data);
+		else 
+			path_list_free (gfl_data->files);
+		get_file_list_data_free (gfl_data);
+		return NULL;
+	}
+
+	directory = file_name_from_path ((char*) gfl_data->current_dir->data);
+	if (strcmp (gfl_data->base_dir, "/") == 0)
+        	path = g_strconcat (gfl_data->base_dir, directory, NULL);
+	else
+		path = g_strconcat (gfl_data->base_dir, "/", directory, NULL);
+
+	visit_dir_async (gfl_data->handle,
+			 path,
+			 "*",
+			 TRUE,
+			 TRUE,
+			 TRUE,
+			 FALSE,
+			 FALSE,
+			 FALSE,
+			 gfl_data->include_directories,
+			 get_items_file_list_cb,
+			 gfl_data);
+	g_free (path);
+
+	return gfl_data->handle;
+}
+
+
+VisitDirHandle *
+get_items_file_list_async (GList            *item_list,
+			   const char       *base_dir,
+			   gboolean          include_directories,
+			   VisitDirDoneFunc  done_func,
+			   gpointer          done_data)
+{
+	GetFileListData *gfl_data;
+	int              base_len;
+	GList           *scan;
+
+	g_return_val_if_fail (base_dir != NULL, NULL);
+
+	gfl_data = g_new0 (GetFileListData, 1);
+	
+	base_len = 0;
+	if (strcmp (base_dir, "/") != 0)
+		base_len = strlen (base_dir);
+	
+	for (scan = item_list; scan; scan = scan->next) {
+		char *path = scan->data;
+
+		if (path_is_file (path)) {
+			char *rel_path = g_strdup (path + base_len + 1);
+			gfl_data->files = g_list_prepend (gfl_data->files, rel_path);
+		} else if (path_is_dir (path))
+			gfl_data->dir_list = g_list_prepend (gfl_data->dir_list, g_strdup (path));
+	}
+
+	gfl_data->current_dir = gfl_data->dir_list;
+	gfl_data->base_dir = g_strdup (base_dir);
+	gfl_data->include_directories = include_directories;
+	gfl_data->done_func = done_func;
+	gfl_data->done_data = done_data;
+	gfl_data->handle = g_new0 (VisitDirHandle, 1);
+
+	return visit_current_dir (gfl_data);
+}
