@@ -50,6 +50,32 @@
 #define BUF_SIZE 4096
 
 
+gboolean
+path_exists (const gchar *path)
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult result;
+	gboolean exists;
+	gchar *escaped;
+
+	if (! path || ! *path) return FALSE; 
+
+	info = gnome_vfs_file_info_new ();
+	escaped = gnome_vfs_escape_path_string (path);
+	result = gnome_vfs_get_file_info (escaped, 
+					  info, 
+					  (GNOME_VFS_FILE_INFO_DEFAULT 
+					   | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
+
+	exists = (result == GNOME_VFS_OK);		
+	
+	g_free (escaped);
+	gnome_vfs_file_info_unref (info);
+
+	return exists;
+}
+
+
 gboolean
 path_is_file (const gchar *path)
 {
@@ -161,6 +187,13 @@ dir_contains_one_object (const char *path)
 }
 
 
+gboolean 
+dir_is_temp_dir (const char *dir) {
+	const char *tmp_dir = g_get_tmp_dir ();
+	return (strncmp (dir, tmp_dir, strlen (tmp_dir)) == 0);
+}
+
+
 /* Check whether the path_src is contained in path_dest */
 gboolean
 path_in_path (const char  *path_src,
@@ -358,6 +391,30 @@ file_name_from_path (const gchar *file_name)
 }
 
 
+char *
+dir_name_from_path (const gchar *path)
+{
+	register  gssize base;
+	register  gssize last_char;
+
+	if (path == NULL) 
+		return NULL;
+
+	if (path[0] == '\0')
+		return g_strdup ("");
+
+	last_char = strlen (path) - 1;
+	if (path[last_char] == G_DIR_SEPARATOR) 
+		last_char--;
+
+	base = last_char;
+	while ((base >= 0) && (path[base] != G_DIR_SEPARATOR))
+		base--;	
+	
+	return g_strndup (path + base + 1, last_char - base);
+}
+
+
 gchar *
 remove_level_from_path (const gchar *path)
 {
@@ -620,8 +677,10 @@ char *
 get_temp_work_dir (void)
 {
 	char temp_dir_template[] = "/tmp/fr-XXXXXX";
-	g_assert (mkdtemp (temp_dir_template) != NULL);
-	return g_strdup (temp_dir_template);
+	if (mkdtemp (temp_dir_template) == NULL)
+		return NULL;
+	else
+		return g_strdup (temp_dir_template);
 }
 
 
