@@ -125,7 +125,8 @@ FRCommandDescription command_desc[] = {
 	{ "unstuff",    TRUE,  FALSE, FALSE, FR_FILE_TYPE_STUFFIT },
 	{ "zip",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_WAR },
 	{ "zoo",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_ZOO },
-	{ "7za",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP }
+	{ "7za",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP },
+	{ "7zr",        TRUE,  TRUE,  TRUE,  FR_FILE_TYPE_7ZIP }
 };
 
 FRCommandDescription tar_command_desc[] = {
@@ -347,7 +348,7 @@ get_path_from_url (char *url)
 	gnome_vfs_uri_unref (uri);
 	path = gnome_vfs_unescape_string (escaped, NULL);
 	g_free (escaped);
-
+	
 	return path;
 }
 
