@@ -42,24 +42,28 @@ static GtkActionEntry action_entries[] = {
 	  NULL, NULL,
 	  N_("Information about the program"),
 	  G_CALLBACK (activate_action_about) },
-	{ "AddFiles", FR_STOCK_ADD,
+	{ "AddFiles", FR_STOCK_ADD_FILES,
 	  N_("_Add Files..."), NULL,
 	  N_("Add files to the archive"),
 	  G_CALLBACK (activate_action_add_files) },
-	{ "AddFiles_Toolbar", FR_STOCK_ADD,
-	  N_("Add"), NULL,
+	{ "AddFiles_Toolbar", FR_STOCK_ADD_FILES,
+	  N_("Add Files"), NULL,
 	  N_("Add files to the archive"),
 	  G_CALLBACK (activate_action_add_files) },
-	{ "AddFolder", FR_STOCK_ADD,
+	{ "AddFolder", FR_STOCK_ADD_FOLDER,
 	  N_("Add a _Folder..."), NULL,
 	  N_("Add a folder to the archive"),
 	  G_CALLBACK (activate_action_add_folder) },
+	{ "AddFolder_Toolbar", FR_STOCK_ADD_FOLDER,
+	  N_("Add Folder"), NULL,
+	  N_("Add a folder to the archive"),
+	  G_CALLBACK (activate_action_add_folder) },	  
 	{ "Close", GTK_STOCK_CLOSE,
 	  NULL, NULL,
 	  N_("Close the current archive"),
 	  G_CALLBACK (activate_action_close) },
 	{ "Contents", GTK_STOCK_HELP,
-	  NULL, "F1",
+	  N_("Contents"), "F1",
 	  N_("Display the File Roller Manual"),
 	  G_CALLBACK (activate_action_manual) },
 	{ "Copy", GTK_STOCK_COPY,
@@ -147,13 +151,17 @@ static GtkActionEntry action_entries[] = {
 	  N_("Test whether the archive contains errors"),
 	  G_CALLBACK (activate_action_test_archive) },
 	{ "ViewSelection", FR_STOCK_VIEW,
-	  N_("_View"), NULL,
+	  NULL, NULL,
 	  N_("View the selected file"),
 	  G_CALLBACK (activate_action_view_or_open) },
 	{ "ViewSelection_Toolbar", FR_STOCK_VIEW,
-	  N_("_View"), NULL,
+	  NULL, NULL,
 	  N_("View the selected file"),
 	  G_CALLBACK (activate_action_view_or_open) },
+	{ "OpenFolder", GTK_STOCK_OPEN,
+	  NULL, NULL,
+	  N_("Open the selected folder"),
+	  G_CALLBACK (activate_action_open_folder) },
 };
 static guint n_action_entries = G_N_ELEMENTS (action_entries);
 
@@ -272,16 +280,18 @@ static const gchar *ui_info =
 "  </menubar>"
 "  <toolbar  name='ToolBar'>"
 "    <toolitem action='New'/>"
+"    <toolitem action='Extract_Toolbar'/>"
 "    <separator/>"
 "    <toolitem action='AddFiles_Toolbar'/>"
-"    <toolitem action='Extract_Toolbar'/>"
+"    <toolitem action='AddFolder_Toolbar'/>"
 "    <toolitem action='ViewSelection_Toolbar'/>"
 "    <separator/>"
 "    <toolitem action='Stop'/>"
 "  </toolbar>"
-"  <popup name='ListPopupMenu'>"
+"  <popup name='FilePopupMenu'>"
 "    <menuitem action='ViewSelection'/>"
 "    <menuitem action='OpenSelection'/>"
+"    <separator/>"
 "    <menuitem action='Extract'/>"
 "    <separator/>"
 "    <menuitem action='Cut'/>"
@@ -290,6 +300,22 @@ static const gchar *ui_info =
 "    <menuitem action='Rename'/>"
 "    <menuitem action='Delete'/>"
 "  </popup>"
+"  <popup name='FolderPopupMenu'>"
+"    <menuitem action='OpenFolder'/>"
+"    <separator/>"
+"    <menuitem action='Extract'/>"
+"    <separator/>"
+"    <menuitem action='Cut'/>"
+"    <menuitem action='Copy'/>"
+"    <menuitem action='Paste'/>"
+"    <menuitem action='Rename'/>"
+"    <menuitem action='Delete'/>"
+"  </popup>"
+"  <popup name='AddMenu'>"
+"    <menuitem action='AddFiles'/>"
+"    <menuitem action='AddFolder'/>"
+"  </popup>"
+
 "</ui>";
 
 
